#!/usr/bin/perl -w

print "<?xml version=\"1.0\"?>\n";
print "<nodeinfo>\n";
print xmlencode("hostname",`hostname`),"\n";
print xmlencode("date",`date`),"\n";
print xmlencode("time",time()),"\n";
print xmlencodeFile("redhat","/etc/redhat-release"),"\n";
print xmlencode("linux",`uname -r`),"\n";
print xmlencodeFile("cpuinfo","/proc/cpuinfo"),"\n";
print xmlencodeFile("meminfo","/proc/meminfo"),"\n";
print xmlencode("uptime",`uptime`),"\n";
print xmlencode("chkconfig",`/sbin/chkconfig --list | grep on`),"\n";
print xmlencode("etchosts",`grep -v ^# /etc/hosts`),"\n";
print xmlencode("etcresolvconf",`grep -v ^# /etc/resolv.conf`),"\n";
my $domainname = `domainname`; chop $domainname;
if ($domainname ne "(none)")
  {
    print xmlencode("domainname",`domainname`),"\n";
    print xmlencode("etcypconf",`grep -v ^# /etc/yp.conf`),"\n";
    print xmlencode("ypwhich",`ypwhich`),"\n";
  }
print xmlencodeFile("ifcfgeth0","/etc/sysconfig/network-scripts/ifcfg-eth0"),"\n";
print xmlencode("miitool",`/sbin/mii-tool`),"\n";
print xmlencodeFile("printcap","/etc/printcap"),"\n";
if (-x "/usr/bin/ntpstat")
  {
    print xmlencode("ntpstat",`/usr/bin/ntpstat`),"\n";
  }
elsif (-x "/usr/sbin/ntptrace")
  {
    print xmlencode("ntp",`/usr/sbin/ntptrace`),"\n";
  }
print xmlencode("mailspool",`/bin/ls -l /var/spool/mail`),"\n";
print xmlencode("ide",encodeIde()),"\n";
print encodeSmart();
print xmlencodeFile("scsi","/proc/scsi/scsi"),"\n";
print xmlencodeFile("etcnodeinfo","/etc/nodeinfo"),"\n";
print xmlencode("lspci",`/sbin/lspci`),"\n";
print xmlencodeFile("rootforward","/root/.forward"),"\n";
print xmlencodeFile("hostsallow","/etc/hosts.allow"),"\n";
print xmlencodeFile("hostsdeny", "/etc/hosts.deny"),"\n";
print xmlencodeFile("securenets","/var/yp/securenets"),"\n";
print xmlencode("rpmnew",`/usr/bin/locate rpmnew`),"\n";
print xmlencode("rpmsave",`/usr/bin/locate rpmsave`),"\n";
print xmlencode("rpms",`rpm -q -a | sort -u`),"\n";
print "</nodeinfo>\n";

exit 0;

sub xmlencodeFile
  {
    my $tag  = shift @_;
    my $file = shift @_;
    return if ! -e $file;
    open(TMPIN,$file) || return "";
    my $contents = join("",<TMPIN>);
    close TMPIN;
    $contents =~ s/\n$//;
    return "<$tag>$contents</$tag>";
  }

sub xmlencode
  {
    my $tag = shift @_;
    my $contents = join("",@_);
    $contents =~ s/\n$//;
    return "<$tag>$contents</$tag>";
  }

sub encodeIde
  {
    my $ret = "";
    foreach my $drive (split(/\n/,`/bin/ls -1 /proc/ide`))
      {
	next if ! -r "/proc/ide/$drive/capacity";
	my $cap   = `cat /proc/ide/$drive/capacity`; chop $cap;
	my $model = `cat /proc/ide/$drive/model`; chop $model;
	my $cache = "none";
	if (-r "/proc/ide/$drive/cache")
	  {
	    $cache = `cat /proc/ide/$drive/cache`; chop $cache;
	  }
	$ret .= "$drive:$cap:$cache:$model\n";
      }
    return $ret;
  }

sub encodeSmart
  {
    my $smartctl = "/usr/sbin/smartctl";

    return "" if ! -x $smartctl;

    my $xml = "";

    $xml .= "<smart>\n";
    
    foreach my $dev ("/dev/hda", "/dev/hdb", "/dev/hdc", "/dev/hdd")
      {
        my $data = `$smartctl -a $dev 2>&1`;
        if ($data =~ /Use option -e to enable/)
          {
            system "$smartctl -e $dev >/dev/null 2>&1";
            $data = `$smartctl -a $dev 2>&1`;
          }
        if ($data =~ /Use option -s with argument 'on' to enable it/)
          {
            system "$smartctl -s on $dev >/dev/null 2>&1";
            $data = `$smartctl -a $dev 2>&1`;
          }
        
        # check return values from RHL8.0-9 smartctl
        next if $data =~ /No such device or address/; # no device
        next if $data =~ /Read-only file system/; # probably a CDROM
        
        # check return values from Fedora-1 smartctl
        if ($data =~ /-T permissive/)
          {
            $data = `$smartctl -T permissive -a $dev 2>&1`;
          }
        
        next if $data =~ /No such device/; # no device
        next if $data =~ /SMART support is: Unavailable/; # CDROM
        
        $xml .= "<data dev=\"$dev\">\n".$data."</data>\n";
      }
    
    $xml .= "</smart>\n";

    return $xml;
  }

#end file
